"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const is_ci_1 = require("is-ci");
const path = require("path");
const pkgDir = require("pkg-dir");
const readPkg = require("read-pkg");
const hookScript_1 = require("./hookScript");
const is_1 = require("./is");
const hookList = [
    'applypatch-msg',
    'pre-applypatch',
    'post-applypatch',
    'pre-commit',
    'prepare-commit-msg',
    'commit-msg',
    'post-commit',
    'pre-rebase',
    'post-checkout',
    'post-merge',
    'pre-push',
    'pre-receive',
    'update',
    'post-receive',
    'post-update',
    'push-to-checkout',
    'pre-auto-gc',
    'post-rewrite',
    'sendemail-validate'
];
function writeHook(filename) {
    fs.writeFileSync(filename, hookScript_1.default, 'utf-8');
    fs.chmodSync(filename, parseInt('0755', 8));
}
function createHook(filename) {
    // Get name, used for logging
    const name = path.basename(filename);
    // Check if hook exist
    if (fs.existsSync(filename)) {
        const hook = fs.readFileSync(filename, 'utf-8');
        // Migrate
        if (is_1.isGhooks(hook)) {
            console.log(`migrating existing ghooks script: ${name} `);
            return writeHook(filename);
        }
        // Migrate
        if (is_1.isPreCommit(hook)) {
            console.log(`migrating existing pre-commit script: ${name}`);
            return writeHook(filename);
        }
        // Update
        if (is_1.isHusky(hook)) {
            return writeHook(filename);
        }
        // Skip
        console.log(`skipping existing user hook: ${name}`);
        return;
    }
    // Create hook if it doesn't exist
    writeHook(filename);
}
function createHooks(filenames) {
    filenames.forEach(createHook);
}
function canRemove(filename) {
    if (fs.existsSync(filename)) {
        const data = fs.readFileSync(filename, 'utf-8');
        return is_1.isHusky(data);
    }
    return false;
}
function removeHook(filename) {
    fs.unlinkSync(filename);
}
function removeHooks(filenames) {
    filenames.filter(canRemove).forEach(removeHook);
}
function getHooks(gitDir) {
    const gitHooksDir = path.join(gitDir, 'hooks');
    return hookList.map(hookName => path.join(gitHooksDir, hookName));
}
function getConf(huskyDir) {
    const pkg = readPkg.sync(huskyDir);
    const defaults = {
        skipCI: true
    };
    return Object.assign({ defaults }, pkg.husky);
}
function install(gitDir, huskyDir) {
    console.log('husky > setting up git hooks');
    const userDir = pkgDir.sync(path.join(huskyDir, '..'));
    const conf = getConf(userDir);
    if (is_ci_1.default && conf.skipCI) {
        console.log('CI detected, skipping Git hooks installation"');
        return;
    }
    if (userDir === null) {
        console.log("Can't find package.json, skipping Git hooks installation");
        return;
    }
    if (path.join(userDir, '.git') !== gitDir) {
        console.log(`Expecting package.json to be at the same level than .git, skipping Git hooks installation`);
        console.log(`gitDir: ${gitDir}`);
        console.log(`userDir: ${userDir}`);
        return;
    }
    // Create hooks
    const hooks = getHooks(gitDir);
    createHooks(hooks);
    console.log(`husky > done`);
}
exports.install = install;
function uninstall(gitDir, huskyDir) {
    console.log('husky > uninstalling git hooks');
    const userDir = pkgDir.sync(path.join(huskyDir, '..'));
    if (path.join(userDir, '.git') === gitDir) {
        // Remove hooks
        const hooks = getHooks(gitDir);
        removeHooks(hooks);
    }
    console.log('husky > done');
}
exports.uninstall = uninstall;
